@@ -26,6 +26,9 @@
 #    pragma warning(disable : 4232) /* function pointer to dll symbol */
 #endif
 
+const struct aws_byte_cursor g_test_mrap_endpoint =
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("moujmk3izc19y.mrap.accesspoint.s3-global.amazonaws.com");
+
 const struct aws_byte_cursor g_test_body_content_type = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("text/plain");
 const struct aws_byte_cursor g_test_s3_region = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("us-west-2");
 
@@ -169,11 +172,6 @@ static void s_s3_test_meta_request_shutdown(void *user_data) {
     aws_s3_tester_notify_meta_request_shutdown(tester);
 }
 
-/* Wait for the cleanup notification.  This, and the s_s3_test_client_shutdown function are meant to be used for
- * sequential clean up only, and should not overlap with the "finish" callback.  (Both currently use the same
- * mutex/signal.) */
-static void s_s3_tester_wait_for_client_shutdown(struct aws_s3_tester *tester);
-
 /* Notify the tester that a particular clean up step has finished. */
 static void s_s3_test_client_shutdown(void *user_data);
 
@@ -507,7 +505,7 @@ void aws_s3_tester_clean_up(struct aws_s3_tester *tester) {
     AWS_PRECONDITION(tester);
 
     if (tester->bound_to_client) {
-        s_s3_tester_wait_for_client_shutdown(tester);
+        aws_s3_tester_wait_for_client_shutdown(tester);
         tester->bound_to_client = false;
     }
 
@@ -834,7 +832,7 @@ static bool s_s3_tester_has_client_shutdown(void *user_data) {
     return tester->synced_data.client_shutdown > 0;
 }
 
-static void s_s3_tester_wait_for_client_shutdown(struct aws_s3_tester *tester) {
+void aws_s3_tester_wait_for_client_shutdown(struct aws_s3_tester *tester) {
     AWS_PRECONDITION(tester);
 
     aws_s3_tester_lock_synced_data(tester);
@@ -1172,8 +1170,12 @@ int aws_s3_tester_send_meta_request_with_options(
             bucket_name = &g_test_bucket_name;
         }
 
-        struct aws_string *host_name = aws_s3_tester_build_endpoint_string(allocator, bucket_name, &g_test_s3_region);
-
+        struct aws_string *host_name = NULL;
+        if (options->mrap_test) {
+            host_name = aws_string_new_from_cursor(allocator, &g_test_mrap_endpoint);
+        } else {
+            host_name = aws_s3_tester_build_endpoint_string(allocator, bucket_name, &g_test_s3_region);
+        }
         if (meta_request_options.type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT ||
             (meta_request_options.type == AWS_S3_META_REQUEST_TYPE_DEFAULT &&
              options->default_type_options.mode == AWS_S3_TESTER_DEFAULT_TYPE_MODE_GET)) {
