@@ -309,6 +309,9 @@ int EC_KEY_check_key(const EC_KEY *eckey) {
   }
 
   // Check the public and private keys match.
+  //
+  // NOTE: this is a FIPS pair-wise consistency check for the ECDH case. See SP
+  // 800-56Ar3, page 36.
   if (eckey->priv_key != NULL) {
     EC_RAW_POINT point;
     if (!ec_point_mul_scalar_base(eckey->group, &point,
@@ -347,9 +350,9 @@ static int EVP_EC_KEY_check_fips(EC_KEY *key) {
       !EVP_DigestSign(ctx, sig_der, &sign_len, msg, msg_len)) {
     goto err;
   }
-  #if defined(BORINGSSL_FIPS_BREAK_ECDSA_PWCT)
+  if (boringssl_fips_break_test("ECDSA_PWCT")) {
     msg[0] = ~msg[0];
-  #endif
+  }
   if (!EVP_DigestVerifyInit(ctx, NULL, hash, NULL, evp_pkey) ||
       !EVP_DigestVerify(ctx, sig_der, sign_len, msg, msg_len)) {
     goto err;
@@ -421,7 +424,7 @@ int EC_KEY_check_fips(const EC_KEY *key) {
 end:
   FIPS_service_indicator_unlock_state();
   if(ret){
-    FIPS_service_indicator_update_state();
+    EC_KEY_keygen_verify_service_indicator((EC_KEY*)key);
   }
   return ret;
 }
@@ -516,6 +519,9 @@ int EC_KEY_generate_key_fips(EC_KEY *eckey) {
   // We have to verify both |EC_KEY_generate_key| and |EC_KEY_check_fips| both
   // succeed before updating the indicator state, so we lock the state here.
   FIPS_service_indicator_lock_state();
+
+  boringssl_ensure_ecc_self_test();
+
   do {
     ret = EC_KEY_generate_key(eckey);
     ret &= EC_KEY_check_fips(eckey);
@@ -524,7 +530,7 @@ int EC_KEY_generate_key_fips(EC_KEY *eckey) {
 
   FIPS_service_indicator_unlock_state();
   if (ret) {
-    FIPS_service_indicator_update_state();
+    EC_KEY_keygen_verify_service_indicator(eckey);
     return 1;
   }
 
