@@ -26,6 +26,7 @@
 
 #include "internal.h"
 #include "../bn/internal.h"
+#include "../cpucap/internal.h"
 #include "../../internal.h"
 #include "../../test/abi_test.h"
 #include "../../test/file_test.h"
@@ -103,7 +104,7 @@ TEST(P256_NistzTest, SelectW7) {
 
 TEST(P256_NistzTest, BEEU) {
 #if defined(OPENSSL_X86_64)
-  if ((OPENSSL_ia32cap_P[1] & (1 << 28)) == 0) {
+  if (!CRYPTO_is_AVX_capable()) {
     // No AVX support; cannot run the BEEU code.
     return;
   }
