@@ -57,11 +57,15 @@
 #include <string.h>
 
 #include "../../internal.h"
+#include "../cpucap/internal.h"
 
 #if defined(__cplusplus)
 extern "C" {
 #endif
 
+// The maximum permitted number of cipher blocks per data unit in XTS mode.
+// Reference IEEE Std 1619-2018.
+#define XTS_MAX_BLOCKS_PER_DATA_UNIT            (1<<20)
 
 // block128_f is the type of an AES block cipher implementation.
 //
@@ -161,6 +165,19 @@ typedef struct {
   unsigned mres, ares;
 } GCM128_CONTEXT;
 
+typedef struct xts128_context {
+  AES_KEY *key1, *key2;
+  block128_f block1, block2;
+} XTS128_CONTEXT;
+
+typedef struct {
+  union {
+    double align;
+    AES_KEY ks;
+  } ks1, ks2;  // AES key schedules to use
+  XTS128_CONTEXT xts;
+} EVP_AES_XTS_CTX;
+
 #if defined(OPENSSL_X86) || defined(OPENSSL_X86_64)
 // crypto_gcm_clmul_enabled returns one if the CLMUL implementation of GCM is
 // used.
@@ -253,10 +270,6 @@ void gcm_gmult_clmul(uint64_t Xi[2], const u128 Htable[16]);
 void gcm_ghash_clmul(uint64_t Xi[2], const u128 Htable[16], const uint8_t *inp,
                      size_t len);
 
-OPENSSL_INLINE char gcm_ssse3_capable(void) {
-  return (OPENSSL_ia32cap_P[1] & (1 << (41 - 32))) != 0;
-}
-
 // |gcm_gmult_ssse3| and |gcm_ghash_ssse3| require |Htable| to be
 // 16-byte-aligned, but |gcm_init_ssse3| does not.
 void gcm_init_ssse3(u128 Htable[16], const uint64_t Xi[2]);
@@ -264,7 +277,7 @@ void gcm_gmult_ssse3(uint64_t Xi[2], const u128 Htable[16]);
 void gcm_ghash_ssse3(uint64_t Xi[2], const u128 Htable[16], const uint8_t *in,
                      size_t len);
 
-#if defined(OPENSSL_X86_64)
+#if defined(OPENSSL_X86_64) && !defined(MY_ASSEMBLER_IS_TOO_OLD_FOR_AVX)
 #define GHASH_ASM_X86_64
 void gcm_init_avx(u128 Htable[16], const uint64_t Xi[2]);
 void gcm_gmult_avx(uint64_t Xi[2], const u128 Htable[16]);
@@ -376,6 +389,14 @@ size_t CRYPTO_cts128_encrypt_block(const uint8_t *in, uint8_t *out, size_t len,
                                    const AES_KEY *key, uint8_t ivec[16],
                                    block128_f block);
 
+// XTS.
+
+// CRYPTO_xts128_encrypt encrypts (or decrypts, if |enc| is zero) |len| bytes
+// from |in| to |out| using the given IV in XTS mode. There's no requirement
+// that |len| be a multiple of any value.
+size_t CRYPTO_xts128_encrypt(const XTS128_CONTEXT *ctx,
+                             const uint8_t iv[16], const uint8_t *inp,
+                             uint8_t *out, size_t len, int enc);
 
 // POLYVAL.
 //
