@@ -14,18 +14,27 @@
  */
 
 #include <sys/param.h>
+#include <sys/socket.h>
 #include <netdb.h>
 
 #include <unistd.h>
 #include <getopt.h>
 #include <fcntl.h>
 
-#include <s2n.h>
+#ifndef S2N_INTERN_LIBCRYPTO
+#include <openssl/crypto.h>
+#include <openssl/err.h>
+#endif
+
+#include "api/s2n.h"
 #include "common.h"
-#include <error/s2n_errno.h>
+#include "error/s2n_errno.h"
 
 #include "tls/s2n_connection.h"
 
+#define OPT_TICKET_IN 1000
+#define OPT_TICKET_OUT 1001
+
 void usage()
 {
     fprintf(stderr, "usage: s2nc [options] host [port]\n");
@@ -38,6 +47,8 @@ void usage()
     fprintf(stderr, "  -c [version_string]\n");
     fprintf(stderr, "  --ciphers [version_string]\n");
     fprintf(stderr, "    Set the cipher preference version string. Defaults to \"default\". See USAGE-GUIDE.md\n");
+    fprintf(stderr, "  --enter-fips-mode\n");
+    fprintf(stderr, "    Enter libcrypto's FIPS mode. The linked version of OpenSSL must be built with the FIPS module.\n");
     fprintf(stderr, "  -e,--echo\n");
     fprintf(stderr, "    Listen to stdin after TLS Connection is established and echo it to the Server\n");
     fprintf(stderr, "  -h,--help\n");
@@ -64,6 +75,10 @@ void usage()
     fprintf(stderr, "    Drop and re-make the connection using Session ticket. If session ticket is disabled, then re-make the connection using Session-ID \n");
     fprintf(stderr, "  -T,--no-session-ticket \n");
     fprintf(stderr, "    Disable session ticket for resumption.\n");
+    fprintf(stderr, "  --ticket-out [file path]\n");
+    fprintf(stderr, "    Path to a file where the session ticket can be stored.\n");
+    fprintf(stderr, "  --ticket-in [file path]\n");
+    fprintf(stderr, "    Path to session ticket file to resume connection.\n");
     fprintf(stderr, "  -D,--dynamic\n");
     fprintf(stderr, "    Set dynamic record resize threshold\n");
     fprintf(stderr, "  -t,--timeout\n");
@@ -160,7 +175,7 @@ static void setup_s2n_config(struct s2n_config *config, const char *cipher_prefs
                     fprintf(stderr, "Error allocating memory\n");
                     exit(1);
                 }
-                memcpy(protocols[idx], next, length);
+                memmove(protocols[idx], next, length);
                 protocols[idx][length] = '\0';
                 length = 0;
                 idx++;
@@ -177,7 +192,7 @@ static void setup_s2n_config(struct s2n_config *config, const char *cipher_prefs
                 fprintf(stderr, "Error allocating memory\n");
                 exit(1);
             }
-            memcpy(protocols[idx], next, length);
+            memmove(protocols[idx], next, length);
             protocols[idx][length] = '\0';
         }
 
@@ -228,6 +243,8 @@ int main(int argc, char *const *argv)
     const char *client_key = NULL;
     bool client_cert_input = false;
     bool client_key_input = false;
+    const char *ticket_out = NULL;
+    char *ticket_in = NULL;
     uint16_t mfl_value = 0;
     uint8_t insecure = 0;
     int reconnect = 0;
@@ -237,6 +254,7 @@ int main(int argc, char *const *argv)
     uint8_t dyn_rec_timeout = 0;
     /* required args */
     const char *cipher_prefs = "default";
+    int fips_mode = 0;
     const char *host = NULL;
     struct verify_data unsafe_verify_data;
     const char *port = "443";
@@ -252,6 +270,7 @@ int main(int argc, char *const *argv)
     static struct option long_options[] = {
         {"alpn", required_argument, 0, 'a'},
         {"ciphers", required_argument, 0, 'c'},
+        {"enter-fips-mode", no_argument, NULL, 'F'},
         {"echo", no_argument, 0, 'e'},
         {"help", no_argument, 0, 'h'},
         {"name", required_argument, 0, 'n'},
@@ -263,6 +282,8 @@ int main(int argc, char *const *argv)
         {"key", required_argument, 0, 'k'},
         {"insecure", no_argument, 0, 'i'},
         {"reconnect", no_argument, 0, 'r'},
+        {"ticket-out", required_argument, 0, OPT_TICKET_OUT},
+        {"ticket-in", required_argument, 0, OPT_TICKET_IN},
         {"no-session-ticket", no_argument, 0, 'T'},
         {"dynamic", required_argument, 0, 'D'},
         {"timeout", required_argument, 0, 't'},
@@ -291,6 +312,9 @@ int main(int argc, char *const *argv)
         case 'c':
             cipher_prefs = optarg;
             break;
+        case 'F':
+            fips_mode = 1;
+            break;
         case 'e':
             echo_input = 1;
             break;
@@ -326,6 +350,12 @@ int main(int argc, char *const *argv)
         case 'r':
             reconnect = 5;
             break;
+        case OPT_TICKET_OUT:
+            ticket_out = optarg;
+            break;
+        case OPT_TICKET_IN:
+            ticket_in = optarg;
+            break;
         case 'T':
             session_ticket = 0;
             break;
@@ -393,6 +423,23 @@ int main(int argc, char *const *argv)
         exit(1);
     }
 
+    if (fips_mode) {
+#ifndef S2N_INTERN_LIBCRYPTO
+#if defined(OPENSSL_FIPS) || defined(OPENSSL_IS_AWSLC)
+        if (FIPS_mode_set(1) == 0) {
+            unsigned long fips_rc = ERR_get_error();
+            char ssl_error_buf[256]; /* Openssl claims you need no more than 120 bytes for error strings */
+            fprintf(stderr, "s2nc failed to enter FIPS mode with RC: %lu; String: %s\n", fips_rc, ERR_error_string(fips_rc, ssl_error_buf));
+            exit(1);
+        }
+        printf("s2nc entered FIPS mode\n");
+#else
+        fprintf(stderr, "Error entering FIPS mode. s2nc was not built against a FIPS-capable libcrypto.\n");
+        exit(1);
+#endif
+#endif
+    }
+
     GUARD_EXIT(s2n_init(), "Error running s2n_init()");
 
     if ((r = getaddrinfo(host, port, &hints, &ai_list)) != 0) {
@@ -444,6 +491,7 @@ int main(int argc, char *const *argv)
         }
 
         if (ca_file || ca_dir) {
+            GUARD_EXIT(s2n_config_wipe_trust_store(config), "Error wiping trust store");
             if (s2n_config_set_verification_ca_location(config, ca_file, ca_dir) < 0) {
                 print_s2n_error("Error setting CA file for trust store.");
             }
@@ -490,6 +538,15 @@ int main(int argc, char *const *argv)
             GUARD_EXIT(s2n_connection_use_corked_io(conn), "Error setting corked io");
         }
 
+        /* Read in session ticket from previous session */
+        if (ticket_in) {
+            GUARD_EXIT(get_file_size(ticket_in, &session_state_length), "Failed to read ticket-in file");
+            free(session_state);
+            session_state = calloc(session_state_length, sizeof(uint8_t));
+            GUARD_EXIT_NULL(session_state);
+            GUARD_EXIT(load_file_to_array(ticket_in, session_state, session_state_length), "Failed to read ticket-in file");
+        }
+
         /* Update session state in connection if exists */
         if (session_state_length > 0) {
             GUARD_EXIT(s2n_connection_set_session(conn, session_state, session_state_length), "Error setting session state in connection");
@@ -518,7 +575,7 @@ int main(int argc, char *const *argv)
         printf("Connected to %s:%s\n", host, port);
 
         /* Save session state from connection if reconnect is enabled. */
-        if (reconnect > 0) {
+        if (reconnect > 0 || ticket_out) {
             if (conn->actual_protocol_version >= S2N_TLS13) {
                 if (!session_ticket) {
                     print_s2n_error("s2nc can only reconnect in TLS1.3 with session tickets.");
@@ -539,6 +596,9 @@ int main(int argc, char *const *argv)
                     exit(1);
                 }
             }
+            if(ticket_out) {
+                GUARD_EXIT(write_array_to_file(ticket_out, session_state, session_state_length), "Failed to write to ticket-out file");
+            }
         }
 
         if (dyn_rec_threshold > 0 && dyn_rec_timeout > 0) {
