@@ -13,6 +13,7 @@
  * permissions and limitations under the License.
  */
 
+#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <netdb.h>
@@ -22,10 +23,12 @@
 #include <getopt.h>
 #include <errno.h>
 
+#ifndef S2N_INTERN_LIBCRYPTO
 #include <openssl/crypto.h>
 #include <openssl/err.h>
+#endif
 
-#include <s2n.h>
+#include "api/s2n.h"
 #include "common.h"
 
 #include "utils/s2n_safety.h"
@@ -489,7 +492,8 @@ int main(int argc, char *const *argv)
     }
 
     if (fips_mode) {
-#ifdef OPENSSL_FIPS
+#ifndef S2N_INTERN_LIBCRYPTO
+#if defined(OPENSSL_FIPS) || defined(OPENSSL_IS_AWSLC)
         if (FIPS_mode_set(1) == 0) {
             unsigned long fips_rc = ERR_get_error();
             char ssl_error_buf[256]; /* Openssl claims you need no more than 120 bytes for error strings */
@@ -498,8 +502,9 @@ int main(int argc, char *const *argv)
         }
         printf("s2nd entered FIPS mode\n");
 #else
-        fprintf(stderr, "Error entering FIPS mode. s2nd is not linked with a FIPS-capable libcrypto.\n");
+        fprintf(stderr, "Error entering FIPS mode. s2nd was not built against a FIPS-capable libcrypto.\n");
         exit(1);
+#endif
 #endif
     }
 
@@ -531,31 +536,31 @@ int main(int argc, char *const *argv)
         struct s2n_cert_chain_and_key *chain_and_key = s2n_cert_chain_and_key_new();
         GUARD_EXIT(s2n_cert_chain_and_key_load_pem(chain_and_key, certificates[i], private_keys[i]), "Error getting certificate/key");
 
+        if (ocsp_response_file_path) {
+            int fd = open(ocsp_response_file_path, O_RDONLY);
+            if (fd < 0) {
+                fprintf(stderr, "Error opening OCSP response file: '%s'\n", strerror(errno));
+                exit(1);
+            }
+
+            struct stat st = {0};
+            if (fstat(fd, &st) < 0) {
+                fprintf(stderr, "Error fstat-ing OCSP response file: '%s'\n", strerror(errno));
+                exit(1);
+            }
+
+            uint8_t *ocsp_response = mmap(0, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
+            if (s2n_cert_chain_and_key_set_ocsp_data(chain_and_key, ocsp_response, st.st_size) < 0) {
+                fprintf(stderr, "Error adding ocsp response: '%s'\n", s2n_strerror(s2n_errno, "EN"));
+                exit(1);
+            }
+
+            close(fd);
+        }
+
         GUARD_EXIT(s2n_config_add_cert_chain_and_key_to_store(config, chain_and_key), "Error setting certificate/key");
     }
 
-    if (ocsp_response_file_path) {
-        int fd = open(ocsp_response_file_path, O_RDONLY);
-        if (fd < 0) {
-            fprintf(stderr, "Error opening OCSP response file: '%s'\n", strerror(errno));
-            exit(1);
-        }
-
-        struct stat st = {0};
-        if (fstat(fd, &st) < 0) {
-            fprintf(stderr, "Error fstat-ing OCSP response file: '%s'\n", strerror(errno));
-            exit(1);
-        }
-
-        uint8_t *ocsp_response = mmap(0, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
-        if (s2n_config_set_extension_data(config, S2N_EXTENSION_OCSP_STAPLING, ocsp_response, st.st_size) < 0) {
-            fprintf(stderr, "Error adding ocsp response: '%s'\n", s2n_strerror(s2n_errno, "EN"));
-            exit(1);
-        }
-
-        close(fd);
-    }
-
     s2n_set_common_server_config(max_early_data, config, conn_settings, cipher_prefs, session_ticket_key_file_path);
 
     if (parallelize) {
