@@ -107,7 +107,11 @@ typedef enum {
     S2N_ERR_NON_EMPTY_RENEGOTIATION_INFO,
     S2N_ERR_RECORD_LIMIT,
     S2N_ERR_CERT_UNTRUSTED,
+    S2N_ERR_CERT_REVOKED,
+    S2N_ERR_CERT_EXPIRED,
     S2N_ERR_CERT_TYPE_UNSUPPORTED,
+    S2N_ERR_CERT_INVALID,
+    S2N_ERR_CERT_MAX_CHAIN_DEPTH_EXCEEDED,
     S2N_ERR_INVALID_MAX_FRAG_LEN,
     S2N_ERR_MAX_FRAG_LEN_MISMATCH,
     S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED,
@@ -209,6 +213,8 @@ typedef enum {
     S2N_ERR_PKEY_CTX_INIT,
     S2N_ERR_FORK_DETECTION_INIT,
     S2N_ERR_RETRIEVE_FORK_GENERATION_NUMBER,
+    S2N_ERR_LIBCRYPTO_VERSION_NUMBER_MISMATCH,
+    S2N_ERR_LIBCRYPTO_VERSION_NAME_MISMATCH,
     S2N_ERR_T_INTERNAL_END,
 
     /* S2N_ERR_T_USAGE */
@@ -284,6 +290,9 @@ typedef enum {
     S2N_ERR_INVALID_X509_EXTENSION_TYPE,
     S2N_ERR_INSUFFICIENT_MEM_SIZE,
     S2N_ERR_KEYING_MATERIAL_EXPIRED,
+    S2N_ERR_SECRET_SCHEDULE_STATE,
+    S2N_ERR_CERT_OWNERSHIP,
+    S2N_ERR_INTERNAL_LIBCRYPTO_ERROR,
     S2N_ERR_T_USAGE_END,
 } s2n_error;
 
