@@ -123,6 +123,10 @@ struct s2n_connection {
     /* Connection successfully set a ticket on the connection */
     unsigned set_session:1;
 
+    /* Buffer multiple records before flushing them.
+     * This allows multiple records to be written with one socket send. */
+    unsigned multirecord_send:1;
+
     /* The configuration (cert, key .. etc ) */
     struct s2n_config *config;
 
@@ -144,7 +148,7 @@ struct s2n_connection {
     void *send_io_context;
     void *recv_io_context;
 
-    /* Track request extensions to ensure correct response extension behavior.
+    /* Track request/response extensions to ensure correct response extension behavior.
      *
      * We need to track client and server extensions separately because some
      * extensions (like request_status and other Certificate extensions) can
@@ -152,6 +156,7 @@ struct s2n_connection {
      */
     s2n_extension_bitfield extension_requests_sent;
     s2n_extension_bitfield extension_requests_received;
+    s2n_extension_bitfield extension_responses_received;
 
     /* Is this connection a client or a server connection */
     s2n_mode mode;
@@ -276,7 +281,8 @@ struct s2n_connection {
     /* Reset record size back to a single segment after threshold seconds of inactivity */
     uint16_t dynamic_record_timeout_threshold;
 
-    /* number of bytes consumed during application activity */
+    /* The number of bytes consumed during a period of application activity.
+     * Used for dynamic record sizing. */
     uint64_t active_application_bytes_consumed;
 
     /* Negotiated TLS extension Maximum Fragment Length code.
@@ -352,7 +358,7 @@ struct s2n_connection {
     struct s2n_blob application_protocols_overridden;
 
     /* Cookie extension data */
-    struct s2n_stuffer cookie_stuffer;
+    struct s2n_blob cookie;
 
     /* Flags to prevent users from calling methods recursively.
      * This can be an easy mistake to make when implementing callbacks.
