@@ -1008,6 +1008,7 @@ static int s2n_handshake_write_io(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_wipe(&conn->handshake.io));
 
     /* Update the secrets, if necessary */
+    POSIX_GUARD_RESULT(s2n_tls13_secrets_update(conn));
     POSIX_GUARD_RESULT(s2n_tls13_key_schedule_update(conn));
 
     /* Advance the state machine */
@@ -1139,6 +1140,7 @@ static S2N_RESULT s2n_finish_read(struct s2n_connection *conn)
 
     RESULT_GUARD_POSIX(s2n_handshake_conn_update_hashes(conn));
     RESULT_GUARD_POSIX(s2n_stuffer_wipe(&conn->handshake.io));
+    RESULT_GUARD(s2n_tls13_secrets_update(conn));
     RESULT_GUARD(s2n_tls13_key_schedule_update(conn));
     RESULT_GUARD_POSIX(s2n_advance_message(conn));
     return S2N_RESULT_OK;
@@ -1363,6 +1365,11 @@ int s2n_negotiate_impl(struct s2n_connection *conn, s2n_blocked_status *blocked)
         /* Flush any pending I/O or alert messages */
         POSIX_GUARD(s2n_flush(conn, blocked));
 
+        /* If the connection is closed, the handshake will never complete. */
+        if (conn->closed) {
+            POSIX_BAIL(S2N_ERR_CLOSED);
+        }
+
         /* If the handshake was paused, retry the current message */
         if (conn->handshake.paused) {
             *blocked = S2N_BLOCKED_ON_APPLICATION_INPUT;
@@ -1424,7 +1431,8 @@ int s2n_negotiate_impl(struct s2n_connection *conn, s2n_blocked_status *blocked)
         }
 
         if (ACTIVE_STATE(conn).writer == 'B') {
-            POSIX_GUARD_RESULT(s2n_tls13_secrets_finish(conn));
+            /* Clean up handshake secrets */
+            POSIX_GUARD_RESULT(s2n_tls13_secrets_clean(conn));
 
             /* Send any pending post-handshake messages */
             POSIX_GUARD(s2n_post_handshake_send(conn, blocked));
