@@ -33,7 +33,7 @@ int s2n_key_update_recv(struct s2n_connection *conn, struct s2n_stuffer *request
 {
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE(conn->actual_protocol_version >= S2N_TLS13, S2N_ERR_BAD_MESSAGE);
-    POSIX_ENSURE(!conn->config->quic_enabled, S2N_ERR_BAD_MESSAGE);
+    POSIX_ENSURE(!s2n_connection_is_quic_enabled(conn), S2N_ERR_BAD_MESSAGE);
 
     uint8_t key_update_request;
     POSIX_GUARD(s2n_stuffer_read_uint8(request, &key_update_request));
@@ -65,6 +65,15 @@ int s2n_key_update_send(struct s2n_connection *conn, s2n_blocked_status *blocked
     POSIX_GUARD(s2n_check_record_limit(conn, &sequence_number));
 
     if (conn->key_update_pending) {
+        /* Flush any buffered records to ensure an empty output buffer.
+         *
+         * This is important when buffering multiple records because we don't:
+         * 1) Respect max fragment length for handshake messages
+         * 2) Check if there is sufficient space in the output buffer for
+         *    post-handshake messages.
+         */
+        POSIX_GUARD(s2n_flush(conn, blocked));
+
         uint8_t key_update_data[S2N_KEY_UPDATE_MESSAGE_SIZE];
         struct s2n_blob key_update_blob = {0};
         POSIX_GUARD(s2n_blob_init(&key_update_blob, key_update_data, sizeof(key_update_data)));
@@ -107,10 +116,20 @@ int s2n_check_record_limit(struct s2n_connection *conn, struct s2n_blob *sequenc
     POSIX_ENSURE_REF(conn->secure.cipher_suite);
     POSIX_ENSURE_REF(conn->secure.cipher_suite->record_alg);
 
-    uint64_t output = 0;
-    POSIX_GUARD(s2n_sequence_number_to_uint64(sequence_number, &output));
+    /*
+     * This is the sequence number that will be used for the next record,
+     * because we incremented the sequence number after sending the last record.
+     */
+    uint64_t next_seq_num = 0;
+    POSIX_GUARD(s2n_sequence_number_to_uint64(sequence_number, &next_seq_num));
 
-    if (output + 1 > conn->secure.cipher_suite->record_alg->encryption_limit) {
+    /*
+     * If the next record is the last record we can send, then the next record needs
+     * to contain a KeyUpdate message.
+     *
+     * This should always trigger on "==", but we use ">=" just in case.
+     */
+    if (next_seq_num >= conn->secure.cipher_suite->record_alg->encryption_limit) {
         conn->key_update_pending = true;
     }
 
