@@ -14,6 +14,7 @@
  */
 
 #include <sys/param.h>
+#include <openssl/hmac.h>
 #include <openssl/md5.h>
 #include <openssl/sha.h>
 #include <string.h>
@@ -43,6 +44,11 @@ static int s2n_sslv3_prf(struct s2n_connection *conn, struct s2n_blob *secret, s
     POSIX_ENSURE_REF(conn->handshake.hashes);
     struct s2n_hash_state *workspace = &conn->handshake.hashes->hash_workspace;
 
+    /* FIPS specifically allows MD5 for the legacy PRF */
+    if (s2n_is_in_fips_mode() && conn->actual_protocol_version < S2N_TLS12) {
+        POSIX_GUARD(s2n_hash_allow_md5_for_fips(workspace));
+    }
+
     uint32_t outputlen = out->size;
     uint8_t *output = out->data;
     uint8_t iteration = 1;
@@ -93,73 +99,78 @@ static int s2n_sslv3_prf(struct s2n_connection *conn, struct s2n_blob *secret, s
     return 0;
 }
 
+static int s2n_init_md_from_hmac_alg(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg){
+    switch (alg) {
+        case S2N_HMAC_SSLv3_MD5:
+        case S2N_HMAC_MD5:
+            ws->p_hash.evp_hmac.evp_digest.md = EVP_md5();
+            break;
+        case S2N_HMAC_SSLv3_SHA1:
+        case S2N_HMAC_SHA1:
+            ws->p_hash.evp_hmac.evp_digest.md = EVP_sha1();
+            break;
+        case S2N_HMAC_SHA224:
+            ws->p_hash.evp_hmac.evp_digest.md = EVP_sha224();
+            break;
+        case S2N_HMAC_SHA256:
+            ws->p_hash.evp_hmac.evp_digest.md = EVP_sha256();
+            break;
+        case S2N_HMAC_SHA384:
+            ws->p_hash.evp_hmac.evp_digest.md = EVP_sha384();
+            break;
+        case S2N_HMAC_SHA512:
+            ws->p_hash.evp_hmac.evp_digest.md = EVP_sha512();
+            break;
+        default:
+            POSIX_BAIL(S2N_ERR_P_HASH_INVALID_ALGORITHM);
+    }
+    return S2N_SUCCESS;
+}
+
 #if !defined(OPENSSL_IS_BORINGSSL) && !defined(OPENSSL_IS_AWSLC)
-static int s2n_evp_hmac_p_hash_new(struct s2n_prf_working_space *ws)
+static int s2n_evp_pkey_p_hash_alloc(struct s2n_prf_working_space *ws)
 {
     POSIX_ENSURE_REF(ws->p_hash.evp_hmac.evp_digest.ctx = S2N_EVP_MD_CTX_NEW());
     return 0;
 }
 
-static int s2n_evp_hmac_p_hash_digest_init(struct s2n_prf_working_space *ws)
+static int s2n_evp_pkey_p_hash_digest_init(struct s2n_prf_working_space *ws)
 {
     POSIX_ENSURE_REF(ws->p_hash.evp_hmac.evp_digest.md);
     POSIX_ENSURE_REF(ws->p_hash.evp_hmac.evp_digest.ctx);
-    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.mac_key);
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.ctx.evp_pkey);
  
     /* Ignore the MD5 check when in FIPS mode to comply with the TLS 1.0 RFC */
     if (s2n_is_in_fips_mode()) {
         POSIX_GUARD(s2n_digest_allow_md5_for_fips(&ws->p_hash.evp_hmac.evp_digest));
     }
 
-    POSIX_GUARD_OSSL(EVP_DigestSignInit(ws->p_hash.evp_hmac.evp_digest.ctx, NULL, ws->p_hash.evp_hmac.evp_digest.md, NULL, ws->p_hash.evp_hmac.mac_key),
+    POSIX_GUARD_OSSL(EVP_DigestSignInit(ws->p_hash.evp_hmac.evp_digest.ctx, NULL, ws->p_hash.evp_hmac.evp_digest.md, NULL, ws->p_hash.evp_hmac.ctx.evp_pkey),
            S2N_ERR_P_HASH_INIT_FAILED);
 
     return 0;
 }
 
-static int s2n_evp_hmac_p_hash_init(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg, struct s2n_blob *secret)
+static int s2n_evp_pkey_p_hash_init(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg, struct s2n_blob *secret)
 {
     /* Initialize the message digest */
-    switch (alg) {
-    case S2N_HMAC_SSLv3_MD5:
-    case S2N_HMAC_MD5:
-        ws->p_hash.evp_hmac.evp_digest.md = EVP_md5();
-        break;
-    case S2N_HMAC_SSLv3_SHA1:
-    case S2N_HMAC_SHA1:
-        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha1();
-        break;
-    case S2N_HMAC_SHA224:
-        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha224();
-        break;
-    case S2N_HMAC_SHA256:
-        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha256();
-        break;
-    case S2N_HMAC_SHA384:
-        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha384();
-        break;
-    case S2N_HMAC_SHA512:
-        ws->p_hash.evp_hmac.evp_digest.md = EVP_sha512();
-        break;
-    default:
-        POSIX_BAIL(S2N_ERR_P_HASH_INVALID_ALGORITHM);
-    }
+    POSIX_GUARD(s2n_init_md_from_hmac_alg(ws, alg));
 
     /* Initialize the mac key using the provided secret */
-    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.mac_key = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, NULL, secret->data, secret->size));
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.ctx.evp_pkey = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, NULL, secret->data, secret->size));
 
     /* Initialize the message digest context with the above message digest and mac key */
-    return s2n_evp_hmac_p_hash_digest_init(ws);
+    return s2n_evp_pkey_p_hash_digest_init(ws);
 }
 
-static int s2n_evp_hmac_p_hash_update(struct s2n_prf_working_space *ws, const void *data, uint32_t size)
+static int s2n_evp_pkey_p_hash_update(struct s2n_prf_working_space *ws, const void *data, uint32_t size)
 {
     POSIX_GUARD_OSSL(EVP_DigestSignUpdate(ws->p_hash.evp_hmac.evp_digest.ctx, data, (size_t)size), S2N_ERR_P_HASH_UPDATE_FAILED);
 
     return 0;
 }
 
-static int s2n_evp_hmac_p_hash_digest(struct s2n_prf_working_space *ws, void *digest, uint32_t size)
+static int s2n_evp_pkey_p_hash_final(struct s2n_prf_working_space *ws, void *digest, uint32_t size)
 {
     /* EVP_DigestSign API's require size_t data structures */
     size_t digest_size = size;
@@ -169,34 +180,41 @@ static int s2n_evp_hmac_p_hash_digest(struct s2n_prf_working_space *ws, void *di
     return 0;
 }
 
-static int s2n_evp_hmac_p_hash_wipe(struct s2n_prf_working_space *ws)
+static int s2n_evp_pkey_p_hash_wipe(struct s2n_prf_working_space *ws)
 {
-  POSIX_GUARD_OSSL(S2N_EVP_MD_CTX_RESET(ws->p_hash.evp_hmac.evp_digest.ctx), S2N_ERR_P_HASH_WIPE_FAILED);
+    POSIX_GUARD_OSSL(S2N_EVP_MD_CTX_RESET(ws->p_hash.evp_hmac.evp_digest.ctx), S2N_ERR_P_HASH_WIPE_FAILED);
 
     return 0;
 }
 
-static int s2n_evp_hmac_p_hash_reset(struct s2n_prf_working_space *ws)
+static int s2n_evp_pkey_p_hash_reset(struct s2n_prf_working_space *ws)
 {
-    POSIX_GUARD(s2n_evp_hmac_p_hash_wipe(ws));
+    POSIX_GUARD(s2n_evp_pkey_p_hash_wipe(ws));
 
-    return s2n_evp_hmac_p_hash_digest_init(ws);
+    /*
+     * On some cleanup paths s2n_evp_pkey_p_hash_reset can be called before s2n_evp_pkey_p_hash_init so there is nothing
+     * to reset.
+     */
+    if (ws->p_hash.evp_hmac.ctx.evp_pkey == NULL) {
+        return S2N_SUCCESS;
+    }
+    return s2n_evp_pkey_p_hash_digest_init(ws);
 }
 
-static int s2n_evp_hmac_p_hash_cleanup(struct s2n_prf_working_space *ws)
+static int s2n_evp_pkey_p_hash_cleanup(struct s2n_prf_working_space *ws)
 {
     /* Prepare the workspace md_ctx for the next p_hash */
-    POSIX_GUARD(s2n_evp_hmac_p_hash_wipe(ws));
+    POSIX_GUARD(s2n_evp_pkey_p_hash_wipe(ws));
 
     /* Free mac key - PKEYs cannot be reused */
-    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.mac_key);
-    EVP_PKEY_free(ws->p_hash.evp_hmac.mac_key);
-    ws->p_hash.evp_hmac.mac_key = NULL;
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.ctx.evp_pkey);
+    EVP_PKEY_free(ws->p_hash.evp_hmac.ctx.evp_pkey);
+    ws->p_hash.evp_hmac.ctx.evp_pkey = NULL;
 
     return 0;
 }
 
-static int s2n_evp_hmac_p_hash_free(struct s2n_prf_working_space *ws)
+static int s2n_evp_pkey_p_hash_free(struct s2n_prf_working_space *ws)
 {
     POSIX_ENSURE_REF(ws->p_hash.evp_hmac.evp_digest.ctx);
     S2N_EVP_MD_CTX_FREE(ws->p_hash.evp_hmac.evp_digest.ctx);
@@ -205,11 +223,74 @@ static int s2n_evp_hmac_p_hash_free(struct s2n_prf_working_space *ws)
     return 0;
 }
 
-static const struct s2n_p_hash_hmac s2n_evp_hmac = {
-    .alloc = &s2n_evp_hmac_p_hash_new,
+static const struct s2n_p_hash_hmac s2n_evp_pkey_p_hash_hmac = {
+    .alloc = &s2n_evp_pkey_p_hash_alloc,
+    .init = &s2n_evp_pkey_p_hash_init,
+    .update = &s2n_evp_pkey_p_hash_update,
+    .final = &s2n_evp_pkey_p_hash_final,
+    .reset = &s2n_evp_pkey_p_hash_reset,
+    .cleanup = &s2n_evp_pkey_p_hash_cleanup,
+    .free = &s2n_evp_pkey_p_hash_free,
+};
+#else
+static int s2n_evp_hmac_p_hash_alloc(struct s2n_prf_working_space *ws)
+{
+    POSIX_ENSURE_REF(ws->p_hash.evp_hmac.ctx.hmac_ctx = HMAC_CTX_new());
+    return S2N_SUCCESS;
+}
+
+static int s2n_evp_hmac_p_hash_init(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg, struct s2n_blob *secret)
+{
+    /* Figure out the correct EVP_MD from s2n_hmac_algorithm  */
+    POSIX_GUARD(s2n_init_md_from_hmac_alg(ws, alg));
+
+    /* Initialize the mac and digest */
+    POSIX_GUARD_OSSL(HMAC_Init_ex(ws->p_hash.evp_hmac.ctx.hmac_ctx, secret->data, secret->size, ws->p_hash.evp_hmac.evp_digest.md, NULL), S2N_ERR_P_HASH_INIT_FAILED);
+    return S2N_SUCCESS;
+}
+
+static int s2n_evp_hmac_p_hash_update(struct s2n_prf_working_space *ws, const void *data, uint32_t size)
+{
+    POSIX_GUARD_OSSL(HMAC_Update(ws->p_hash.evp_hmac.ctx.hmac_ctx, data, (size_t)size), S2N_ERR_P_HASH_UPDATE_FAILED);
+    return S2N_SUCCESS;
+}
+
+static int s2n_evp_hmac_p_hash_final(struct s2n_prf_working_space *ws, void *digest, uint32_t size)
+{
+    /* HMAC_Final API's require size_t data structures */
+    unsigned int digest_size = size;
+    POSIX_GUARD_OSSL(HMAC_Final(ws->p_hash.evp_hmac.ctx.hmac_ctx, (unsigned char *)digest, &digest_size), S2N_ERR_P_HASH_FINAL_FAILED);
+    return S2N_SUCCESS;
+}
+
+static int s2n_evp_hmac_p_hash_reset(struct s2n_prf_working_space *ws)
+{
+    POSIX_ENSURE_REF(ws);
+    if (ws->p_hash.evp_hmac.evp_digest.md == NULL) {
+        return S2N_SUCCESS;
+    }
+    POSIX_GUARD_OSSL(HMAC_Init_ex(ws->p_hash.evp_hmac.ctx.hmac_ctx, NULL, 0, ws->p_hash.evp_hmac.evp_digest.md, NULL), S2N_ERR_P_HASH_INIT_FAILED);
+    return S2N_SUCCESS;
+}
+
+static int s2n_evp_hmac_p_hash_cleanup(struct s2n_prf_working_space *ws)
+{
+    /* Prepare the workspace md_ctx for the next p_hash */
+    HMAC_CTX_reset(ws->p_hash.evp_hmac.ctx.hmac_ctx);
+    return S2N_SUCCESS;
+}
+
+static int s2n_evp_hmac_p_hash_free(struct s2n_prf_working_space *ws)
+{
+    HMAC_CTX_free(ws->p_hash.evp_hmac.ctx.hmac_ctx);
+    return S2N_SUCCESS;
+}
+
+static const struct s2n_p_hash_hmac s2n_evp_hmac_p_hash_hmac = {
+    .alloc = &s2n_evp_hmac_p_hash_alloc,
     .init = &s2n_evp_hmac_p_hash_init,
     .update = &s2n_evp_hmac_p_hash_update,
-    .final = &s2n_evp_hmac_p_hash_digest,
+    .final = &s2n_evp_hmac_p_hash_final,
     .reset = &s2n_evp_hmac_p_hash_reset,
     .cleanup = &s2n_evp_hmac_p_hash_cleanup,
     .free = &s2n_evp_hmac_p_hash_free,
@@ -240,7 +321,13 @@ static int s2n_hmac_p_hash_digest(struct s2n_prf_working_space *ws, void *digest
 
 static int s2n_hmac_p_hash_reset(struct s2n_prf_working_space *ws)
 {
-    return s2n_hmac_reset(&ws->p_hash.s2n_hmac);
+    /* If we actually initialized s2n_hmac, wipe it.
+     * A valid, initialized s2n_hmac_state will have a valid block size.
+     */
+    if (ws->p_hash.s2n_hmac.hash_block_size != 0) {
+        return s2n_hmac_reset(&ws->p_hash.s2n_hmac);
+    }
+    return S2N_SUCCESS;
 }
 
 static int s2n_hmac_p_hash_cleanup(struct s2n_prf_working_space *ws)
@@ -253,7 +340,7 @@ static int s2n_hmac_p_hash_free(struct s2n_prf_working_space *ws)
     return s2n_hmac_free(&ws->p_hash.s2n_hmac);
 }
 
-static const struct s2n_p_hash_hmac s2n_hmac = {
+static const struct s2n_p_hash_hmac s2n_internal_p_hash_hmac = {
     .alloc = &s2n_hmac_p_hash_new,
     .init = &s2n_hmac_p_hash_init,
     .update = &s2n_hmac_p_hash_update,
@@ -263,13 +350,21 @@ static const struct s2n_p_hash_hmac s2n_hmac = {
     .free = &s2n_hmac_p_hash_free,
 };
 
+const struct s2n_p_hash_hmac *s2n_get_hmac_implementation() {
+#if defined(OPENSSL_IS_BORINGSSL) || defined(OPENSSL_IS_AWSLC)
+    return s2n_is_in_fips_mode() ? &s2n_evp_hmac_p_hash_hmac : &s2n_internal_p_hash_hmac;
+#else
+    return s2n_is_in_fips_mode() ? &s2n_evp_pkey_p_hash_hmac : &s2n_internal_p_hash_hmac;
+#endif
+}
+
 static int s2n_p_hash(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg, struct s2n_blob *secret, struct s2n_blob *label,
                       struct s2n_blob *seed_a, struct s2n_blob *seed_b, struct s2n_blob *seed_c, struct s2n_blob *out)
 {
     uint8_t digest_size;
     POSIX_GUARD(s2n_hmac_digest_size(alg, &digest_size));
 
-    const struct s2n_p_hash_hmac *hmac = ws->p_hash_hmac_impl;
+    const struct s2n_p_hash_hmac *hmac = s2n_get_hmac_implementation();
 
     /* First compute hmac(secret + A(0)) */
     POSIX_GUARD(hmac->init(ws, alg, secret));
@@ -323,14 +418,6 @@ static int s2n_p_hash(struct s2n_prf_working_space *ws, s2n_hmac_algorithm alg,
     return 0;
 }
 
-const struct s2n_p_hash_hmac *s2n_get_hmac_implementation() {
-#if defined(OPENSSL_IS_BORINGSSL) || defined(OPENSSL_IS_AWSLC)
-  return &s2n_hmac;
-#else
-  return s2n_is_in_fips_mode() ? &s2n_evp_hmac : &s2n_hmac;
-#endif
-}
-
 S2N_RESULT s2n_prf_new(struct s2n_connection *conn)
 {
     RESULT_ENSURE_REF(conn);
@@ -343,8 +430,8 @@ S2N_RESULT s2n_prf_new(struct s2n_connection *conn)
     ZERO_TO_DISABLE_DEFER_CLEANUP(mem);
 
     /* Allocate the hmac state */
-    conn->prf_space->p_hash_hmac_impl = s2n_get_hmac_implementation();
-    RESULT_GUARD_POSIX(conn->prf_space->p_hash_hmac_impl->alloc(conn->prf_space));
+    const struct s2n_p_hash_hmac *hmac_impl = s2n_get_hmac_implementation();
+    RESULT_GUARD_POSIX(hmac_impl->alloc(conn->prf_space));
     return S2N_RESULT_OK;
 }
 
@@ -353,12 +440,8 @@ S2N_RESULT s2n_prf_wipe(struct s2n_connection *conn)
     RESULT_ENSURE_REF(conn);
     RESULT_ENSURE_REF(conn->prf_space);
 
-    /* If we actually initialized s2n_hmac, wipe it.
-     * A valid, initialized s2n_hmac_state will have a valid block size.
-     */
-    if (conn->prf_space->p_hash.s2n_hmac.hash_block_size != 0) {
-        RESULT_GUARD_POSIX(s2n_hmac_reset(&conn->prf_space->p_hash.s2n_hmac));
-    }
+    const struct s2n_p_hash_hmac *hmac_impl = s2n_get_hmac_implementation();
+    RESULT_GUARD_POSIX(hmac_impl->reset(conn->prf_space));
 
     return S2N_RESULT_OK;
 }
@@ -370,11 +453,8 @@ S2N_RESULT s2n_prf_free(struct s2n_connection *conn)
         return S2N_RESULT_OK;
     }
 
-    /* cppcheck-suppress nullPointerRedundantCheck
-     * cppcheck doesn't recognize the early exit if conn->prf_space == NULL.
-     */
-    conn->prf_space->p_hash_hmac_impl = s2n_get_hmac_implementation();
-    RESULT_GUARD_POSIX(conn->prf_space->p_hash_hmac_impl->free(conn->prf_space));
+    const struct s2n_p_hash_hmac *hmac_impl = s2n_get_hmac_implementation();
+    RESULT_GUARD_POSIX(hmac_impl->free(conn->prf_space));
 
     RESULT_GUARD_POSIX(s2n_free_object((uint8_t **) &conn->prf_space, sizeof(struct s2n_prf_working_space)));
     return S2N_RESULT_OK;
@@ -402,12 +482,7 @@ static int s2n_prf(struct s2n_connection *conn, struct s2n_blob *secret, struct
      * outputs will be XORd just ass the TLS 1.0 and 1.1 RFCs require.
      */
     POSIX_GUARD(s2n_blob_zero(out));
-
-    /* Ensure that p_hash_hmac_impl is set, as it may have been reset for prf_space on s2n_connection_wipe. 
-     * When in FIPS mode, the EVP API's must be used for the p_hash HMAC.
-     */
-    conn->prf_space->p_hash_hmac_impl = s2n_get_hmac_implementation();
-
+    
     if (conn->actual_protocol_version == S2N_TLS12) {
         return s2n_p_hash(conn->prf_space, conn->secure.cipher_suite->prf_alg, secret, label, seed_a, seed_b,
                           seed_c, out);
@@ -424,9 +499,9 @@ static int s2n_prf(struct s2n_connection *conn, struct s2n_blob *secret, struct
 
 int s2n_tls_prf_master_secret(struct s2n_connection *conn, struct s2n_blob *premaster_secret)
 {
-    struct s2n_blob client_random = {.size = sizeof(conn->secrets.client_random), .data = conn->secrets.client_random};
-    struct s2n_blob server_random = {.size = sizeof(conn->secrets.server_random), .data = conn->secrets.server_random};
-    struct s2n_blob master_secret = {.size = sizeof(conn->secrets.master_secret), .data = conn->secrets.master_secret};
+    struct s2n_blob client_random = {.size = sizeof(conn->handshake_params.client_random), .data = conn->handshake_params.client_random};
+    struct s2n_blob server_random = {.size = sizeof(conn->handshake_params.server_random), .data = conn->handshake_params.server_random};
+    struct s2n_blob master_secret = {.size = sizeof(conn->secrets.tls12.master_secret), .data = conn->secrets.tls12.master_secret};
 
     uint8_t master_secret_label[] = "master secret";
     struct s2n_blob label = {.size = sizeof(master_secret_label) - 1, .data = master_secret_label};
@@ -436,9 +511,9 @@ int s2n_tls_prf_master_secret(struct s2n_connection *conn, struct s2n_blob *prem
 
 int s2n_hybrid_prf_master_secret(struct s2n_connection *conn, struct s2n_blob *premaster_secret)
 {
-    struct s2n_blob client_random = {.size = sizeof(conn->secrets.client_random), .data = conn->secrets.client_random};
-    struct s2n_blob server_random = {.size = sizeof(conn->secrets.server_random), .data = conn->secrets.server_random};
-    struct s2n_blob master_secret = {.size = sizeof(conn->secrets.master_secret), .data = conn->secrets.master_secret};
+    struct s2n_blob client_random = {.size = sizeof(conn->handshake_params.client_random), .data = conn->handshake_params.client_random};
+    struct s2n_blob server_random = {.size = sizeof(conn->handshake_params.server_random), .data = conn->handshake_params.server_random};
+    struct s2n_blob master_secret = {.size = sizeof(conn->secrets.tls12.master_secret), .data = conn->secrets.tls12.master_secret};
 
     uint8_t master_secret_label[] = "hybrid master secret";
     struct s2n_blob label = {.size = sizeof(master_secret_label) - 1, .data = master_secret_label};
@@ -498,7 +573,7 @@ int s2n_prf_calculate_master_secret(struct s2n_connection *conn, struct s2n_blob
  */
 S2N_RESULT s2n_tls_prf_extended_master_secret(struct s2n_connection *conn, struct s2n_blob *premaster_secret, struct s2n_blob *session_hash, struct s2n_blob *sha1_hash)
 {
-    struct s2n_blob extended_master_secret = {.size = sizeof(conn->secrets.master_secret), .data = conn->secrets.master_secret};
+    struct s2n_blob extended_master_secret = {.size = sizeof(conn->secrets.tls12.master_secret), .data = conn->secrets.tls12.master_secret};
 
     uint8_t extended_master_secret_label[] = "extended master secret";
     /* Subtract one from the label size to remove the "\0" */
@@ -515,15 +590,14 @@ S2N_RESULT s2n_prf_get_digest_for_ems(struct s2n_connection *conn, struct s2n_bl
     RESULT_ENSURE_REF(conn->handshake.hashes);
     RESULT_ENSURE_REF(output);
 
-    struct s2n_hash_state hash_state = { 0 };
-    RESULT_GUARD_POSIX(s2n_handshake_get_hash_state(conn, hash_alg, &hash_state));
-    RESULT_GUARD_POSIX(s2n_hash_copy(&conn->handshake.hashes->hash_workspace, &hash_state));
-    RESULT_GUARD_POSIX(s2n_hash_update(&conn->handshake.hashes->hash_workspace, message->data, message->size));
+    struct s2n_hash_state *hash_state = &conn->handshake.hashes->hash_workspace;
+    RESULT_GUARD(s2n_handshake_copy_hash_state(conn, hash_alg, hash_state));
+    RESULT_GUARD_POSIX(s2n_hash_update(hash_state, message->data, message->size));
 
     uint8_t digest_size = 0;
-    RESULT_GUARD_POSIX(s2n_hash_digest_size(conn->handshake.hashes->hash_workspace.alg, &digest_size));
+    RESULT_GUARD_POSIX(s2n_hash_digest_size(hash_alg, &digest_size));
     RESULT_ENSURE_GTE(output->size, digest_size);
-    RESULT_GUARD_POSIX(s2n_hash_digest(&conn->handshake.hashes->hash_workspace, output->data, digest_size));
+    RESULT_GUARD_POSIX(s2n_hash_digest(hash_state, output->data, digest_size));
     output->size = digest_size;
 
     return S2N_RESULT_OK;
@@ -550,11 +624,11 @@ static int s2n_sslv3_finished(struct s2n_connection *conn, uint8_t prefix[4], st
     struct s2n_hash_state *md5 = hash_workspace;
     POSIX_GUARD(s2n_hash_copy(md5, &conn->handshake.hashes->md5));
     POSIX_GUARD(s2n_hash_update(md5, prefix, 4));
-    POSIX_GUARD(s2n_hash_update(md5, conn->secrets.master_secret, sizeof(conn->secrets.master_secret)));
+    POSIX_GUARD(s2n_hash_update(md5, conn->secrets.tls12.master_secret, sizeof(conn->secrets.tls12.master_secret)));
     POSIX_GUARD(s2n_hash_update(md5, xorpad1, 48));
     POSIX_GUARD(s2n_hash_digest(md5, md5_digest, MD5_DIGEST_LENGTH));
     POSIX_GUARD(s2n_hash_reset(md5));
-    POSIX_GUARD(s2n_hash_update(md5, conn->secrets.master_secret, sizeof(conn->secrets.master_secret)));
+    POSIX_GUARD(s2n_hash_update(md5, conn->secrets.tls12.master_secret, sizeof(conn->secrets.tls12.master_secret)));
     POSIX_GUARD(s2n_hash_update(md5, xorpad2, 48));
     POSIX_GUARD(s2n_hash_update(md5, md5_digest, MD5_DIGEST_LENGTH));
     POSIX_GUARD(s2n_hash_digest(md5, md5_digest, MD5_DIGEST_LENGTH));
@@ -563,11 +637,11 @@ static int s2n_sslv3_finished(struct s2n_connection *conn, uint8_t prefix[4], st
     struct s2n_hash_state *sha1 = hash_workspace;
     POSIX_GUARD(s2n_hash_copy(sha1, &conn->handshake.hashes->sha1));
     POSIX_GUARD(s2n_hash_update(sha1, prefix, 4));
-    POSIX_GUARD(s2n_hash_update(sha1, conn->secrets.master_secret, sizeof(conn->secrets.master_secret)));
+    POSIX_GUARD(s2n_hash_update(sha1, conn->secrets.tls12.master_secret, sizeof(conn->secrets.tls12.master_secret)));
     POSIX_GUARD(s2n_hash_update(sha1, xorpad1, 40));
     POSIX_GUARD(s2n_hash_digest(sha1, sha_digest, SHA_DIGEST_LENGTH));
     POSIX_GUARD(s2n_hash_reset(sha1));
-    POSIX_GUARD(s2n_hash_update(sha1, conn->secrets.master_secret, sizeof(conn->secrets.master_secret)));
+    POSIX_GUARD(s2n_hash_update(sha1, conn->secrets.tls12.master_secret, sizeof(conn->secrets.tls12.master_secret)));
     POSIX_GUARD(s2n_hash_update(sha1, xorpad2, 40));
     POSIX_GUARD(s2n_hash_update(sha1, sha_digest, SHA_DIGEST_LENGTH));
     POSIX_GUARD(s2n_hash_digest(sha1, sha_digest, SHA_DIGEST_LENGTH));
@@ -619,8 +693,8 @@ int s2n_prf_client_finished(struct s2n_connection *conn)
     label.data = client_finished_label;
     label.size = sizeof(client_finished_label) - 1;
 
-    master_secret.data = conn->secrets.master_secret;
-    master_secret.size = sizeof(conn->secrets.master_secret);
+    master_secret.data = conn->secrets.tls12.master_secret;
+    master_secret.size = sizeof(conn->secrets.tls12.master_secret);
     if (conn->actual_protocol_version == S2N_TLS12) {
         switch (conn->secure.cipher_suite->prf_alg) {
         case S2N_HMAC_SHA256:
@@ -675,8 +749,8 @@ int s2n_prf_server_finished(struct s2n_connection *conn)
     label.data = server_finished_label;
     label.size = sizeof(server_finished_label) - 1;
 
-    master_secret.data = conn->secrets.master_secret;
-    master_secret.size = sizeof(conn->secrets.master_secret);
+    master_secret.data = conn->secrets.tls12.master_secret;
+    master_secret.size = sizeof(conn->secrets.tls12.master_secret);
     if (conn->actual_protocol_version == S2N_TLS12) {
         switch (conn->secure.cipher_suite->prf_alg) {
         case S2N_HMAC_SHA256:
@@ -744,9 +818,9 @@ static int s2n_prf_make_server_key(struct s2n_connection *conn, struct s2n_stuff
 
 int s2n_prf_key_expansion(struct s2n_connection *conn)
 {
-    struct s2n_blob client_random = {.data = conn->secrets.client_random,.size = sizeof(conn->secrets.client_random) };
-    struct s2n_blob server_random = {.data = conn->secrets.server_random,.size = sizeof(conn->secrets.server_random) };
-    struct s2n_blob master_secret = {.data = conn->secrets.master_secret,.size = sizeof(conn->secrets.master_secret) };
+    struct s2n_blob client_random = {.data = conn->handshake_params.client_random,.size = sizeof(conn->handshake_params.client_random) };
+    struct s2n_blob server_random = {.data = conn->handshake_params.server_random,.size = sizeof(conn->handshake_params.server_random) };
+    struct s2n_blob master_secret = {.data = conn->secrets.tls12.master_secret,.size = sizeof(conn->secrets.tls12.master_secret) };
     struct s2n_blob label, out;
     uint8_t key_expansion_label[] = "key expansion";
     uint8_t key_block[S2N_MAX_KEY_BLOCK_LEN];
