@@ -74,11 +74,26 @@ static S2N_RESULT s2n_tls_record_overhead(struct s2n_connection *conn, uint16_t
 S2N_RESULT s2n_record_max_write_payload_size(struct s2n_connection *conn, uint16_t *max_fragment_size)
 {
     RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE_REF(conn->config);
     RESULT_ENSURE_MUT(max_fragment_size);
     RESULT_ENSURE(conn->max_outgoing_fragment_length > 0, S2N_ERR_FRAGMENT_LENGTH_TOO_SMALL);
 
     *max_fragment_size = MIN(conn->max_outgoing_fragment_length, S2N_TLS_MAXIMUM_FRAGMENT_LENGTH);
 
+    /* If a custom send buffer is configured, ensure it will be large enough for the payload.
+     * That may mean we need a smaller fragment size.
+     */
+    uint32_t send_buffer_override = conn->config->send_buffer_size_override;
+    if (send_buffer_override) {
+        uint16_t max_record_size = 0;
+        RESULT_GUARD(s2n_record_max_write_size(conn, *max_fragment_size, &max_record_size));
+        if (send_buffer_override < max_record_size) {
+            size_t overhead = (max_record_size - *max_fragment_size);
+            RESULT_ENSURE_GT(send_buffer_override, overhead);
+            *max_fragment_size = send_buffer_override - overhead;
+        }
+    }
+
     return S2N_RESULT_OK;
 }
 
@@ -143,7 +158,7 @@ S2N_RESULT s2n_record_min_write_payload_size(struct s2n_connection *conn, uint16
     return S2N_RESULT_OK;
 }
 
-int s2n_record_write_protocol_version(struct s2n_connection *conn)
+int s2n_record_write_protocol_version(struct s2n_connection *conn, struct s2n_stuffer *out)
 {
     uint8_t record_protocol_version = conn->actual_protocol_version;
     if (conn->server_protocol_version == s2n_unknown_protocol_version
@@ -164,7 +179,7 @@ int s2n_record_write_protocol_version(struct s2n_connection *conn)
     protocol_version[0] = record_protocol_version / 10;
     protocol_version[1] = record_protocol_version % 10;
 
-    POSIX_GUARD(s2n_stuffer_write_bytes(&conn->out, protocol_version, S2N_TLS_PROTOCOL_VERSION_LEN));
+    POSIX_GUARD(s2n_stuffer_write_bytes(out, protocol_version, S2N_TLS_PROTOCOL_VERSION_LEN));
 
     return 0;
 }
@@ -250,7 +265,10 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
     const int is_tls13_record = cipher_suite->record_alg->flags & S2N_TLS13_RECORD_AEAD_NONCE;
     s2n_stack_blob(aad, is_tls13_record ? S2N_TLS13_AAD_LEN : S2N_TLS_MAX_AAD_LEN, S2N_TLS_MAX_AAD_LEN);
 
-    S2N_ERROR_IF(s2n_stuffer_data_available(&conn->out), S2N_ERR_RECORD_STUFFER_NEEDS_DRAINING);
+    /* If we aren't buffering multiple records, then the output stuffer should be empty. */
+    if (!conn->multirecord_send) {
+        POSIX_ENSURE(s2n_stuffer_data_available(&conn->out) == 0, S2N_ERR_RECORD_STUFFER_NEEDS_DRAINING);
+    }
 
     uint8_t mac_digest_size;
     POSIX_GUARD(s2n_hmac_digest_size(mac->alg, &mac_digest_size));
@@ -279,37 +297,51 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
     POSIX_GUARD(s2n_hmac_update(mac, sequence_number, S2N_TLS_SEQUENCE_NUM_LEN));
 
     if (s2n_stuffer_is_freed(&conn->out)) {
-        /* If the output buffer has not been allocated yet, allocate enough memory to hold
-         * a record with the local maximum fragment length. Because this only occurs if the
-         * output buffer has not been allocated, it does NOT resize existing buffers.
+        /* If the output buffer has not been allocated yet, allocate
+         * at least enough memory to hold a record with the local maximum fragment length.
          *
-         * The maximum fragment length is:
+         * The local maximum fragment length is:
          * 1) The local default configured for new connections
          * 2) The local value set by the user via s2n_connection_prefer_throughput()
          *    or s2n_connection_prefer_low_latency()
          * 3) On the server, the minimum of the local value and the value negotiated with the
          *    client via the max_fragment_length extension
+         *
+         * Because this only occurs if the output buffer has not been allocated,
+         * it does NOT resize existing buffers.
          */
         uint16_t max_wire_record_size = 0;
         POSIX_GUARD_RESULT(s2n_record_max_write_size(conn, max_write_payload_size, &max_wire_record_size));
-        POSIX_GUARD(s2n_stuffer_growable_alloc(&conn->out, max_wire_record_size));
+
+        uint32_t buffer_size = MAX(conn->config->send_buffer_size_override, max_wire_record_size);
+        POSIX_GUARD(s2n_stuffer_growable_alloc(&conn->out, buffer_size));
     }
 
+    /* A record only local stuffer used to avoid tainting the conn->out stuffer or overwriting
+     * previous records. It should be used to add an individual record to the out stuffer.
+     */
+    struct s2n_blob record_blob = { 0 };
+    struct s2n_stuffer record_stuffer = { 0 };
+    POSIX_GUARD(s2n_blob_init(&record_blob,
+            conn->out.blob.data + conn->out.write_cursor,
+            s2n_stuffer_space_remaining(&conn->out)));
+    POSIX_GUARD(s2n_stuffer_init(&record_stuffer, &record_blob));
+
     /* Now that we know the length, start writing the record */
-    POSIX_GUARD(s2n_stuffer_write_uint8(&conn->out, is_tls13_record ?
+    POSIX_GUARD(s2n_stuffer_write_uint8(&record_stuffer, is_tls13_record ?
         /* tls 1.3 opaque type */ TLS_APPLICATION_DATA :
         /* actual content_type */ content_type ));
-    POSIX_GUARD(s2n_record_write_protocol_version(conn));
+    POSIX_GUARD(s2n_record_write_protocol_version(conn, &record_stuffer));
 
     /* First write a header that has the payload length, this is for the MAC */
-    POSIX_GUARD(s2n_stuffer_write_uint16(&conn->out, data_bytes_to_take));
+    POSIX_GUARD(s2n_stuffer_write_uint16(&record_stuffer, data_bytes_to_take));
 
     if (conn->actual_protocol_version > S2N_SSLv3) {
-        POSIX_GUARD(s2n_hmac_update(mac, conn->out.blob.data, S2N_TLS_RECORD_HEADER_LENGTH));
+        POSIX_GUARD(s2n_hmac_update(mac, record_stuffer.blob.data, S2N_TLS_RECORD_HEADER_LENGTH));
     } else {
         /* SSLv3 doesn't include the protocol version in the MAC */
-        POSIX_GUARD(s2n_hmac_update(mac, conn->out.blob.data, 1));
-        POSIX_GUARD(s2n_hmac_update(mac, conn->out.blob.data + 3, 2));
+        POSIX_GUARD(s2n_hmac_update(mac, record_stuffer.blob.data, 1));
+        POSIX_GUARD(s2n_hmac_update(mac, record_stuffer.blob.data + 3, 2));
     }
 
     /* Compute non-payload parts of the MAC(seq num, type, proto vers, fragment length) for composite ciphers.
@@ -339,8 +371,8 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
     /* ensure actual_fragment_length + S2N_TLS_RECORD_HEADER_LENGTH <= max record length */
     const uint16_t max_record_length = is_tls13_record ? S2N_TLS13_MAXIMUM_RECORD_LENGTH : S2N_TLS_MAXIMUM_RECORD_LENGTH;
     S2N_ERROR_IF(actual_fragment_length + S2N_TLS_RECORD_HEADER_LENGTH > max_record_length, S2N_ERR_RECORD_LENGTH_TOO_LARGE);
-    POSIX_GUARD(s2n_stuffer_wipe_n(&conn->out, 2));
-    POSIX_GUARD(s2n_stuffer_write_uint16(&conn->out, actual_fragment_length));
+    POSIX_GUARD(s2n_stuffer_wipe_n(&record_stuffer, 2));
+    POSIX_GUARD(s2n_stuffer_write_uint16(&record_stuffer, actual_fragment_length));
 
     /* If we're AEAD, write the sequence number as an IV, and generate the AAD */
     if (cipher_suite->record_alg->cipher->type == S2N_AEAD) {
@@ -350,7 +382,7 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
 
         if (cipher_suite->record_alg->flags & S2N_TLS12_AES_GCM_AEAD_NONCE) {
             /* Partially explicit nonce. See RFC 5288 Section 3 */
-            POSIX_GUARD(s2n_stuffer_write_bytes(&conn->out, sequence_number, S2N_TLS_SEQUENCE_NUM_LEN));
+            POSIX_GUARD(s2n_stuffer_write_bytes(&record_stuffer, sequence_number, S2N_TLS_SEQUENCE_NUM_LEN));
             POSIX_GUARD(s2n_stuffer_write_bytes(&iv_stuffer, implicit_iv, cipher_suite->record_alg->cipher->io.aead.fixed_iv_size));
             POSIX_GUARD(s2n_stuffer_write_bytes(&iv_stuffer, sequence_number, S2N_TLS_SEQUENCE_NUM_LEN));
         } else if (cipher_suite->record_alg->flags & S2N_TLS12_CHACHA_POLY_AEAD_NONCE || is_tls13_record) {
@@ -396,12 +428,12 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
                 uint8_t zero_block[S2N_TLS_MAX_IV_LEN] = { 0 };
                 POSIX_GUARD(s2n_blob_init(&explicit_iv_placeholder, zero_block, block_size));
                 POSIX_GUARD_RESULT(s2n_get_public_random_data(&explicit_iv_placeholder));
-                POSIX_GUARD(s2n_stuffer_write(&conn->out, &explicit_iv_placeholder));
+                POSIX_GUARD(s2n_stuffer_write(&record_stuffer, &explicit_iv_placeholder));
             } else {
                 /* We can write the explicit IV directly to the record for non composite CBC because
                  * s2n starts AES *after* the explicit IV.
                  */
-                POSIX_GUARD(s2n_stuffer_write(&conn->out, &iv));
+                POSIX_GUARD(s2n_stuffer_write(&record_stuffer, &iv));
             }
         }
     }
@@ -411,12 +443,12 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
     POSIX_GUARD(s2n_increment_sequence_number(&seq));
 
     /* Write the plaintext data */
-    POSIX_GUARD(s2n_stuffer_writev_bytes(&conn->out, in, in_count, offs, data_bytes_to_take));
-    void *orig_write_ptr = conn->out.blob.data + conn->out.write_cursor - data_bytes_to_take;
+    POSIX_GUARD(s2n_stuffer_writev_bytes(&record_stuffer, in, in_count, offs, data_bytes_to_take));
+    void *orig_write_ptr = record_stuffer.blob.data + record_stuffer.write_cursor - data_bytes_to_take;
     POSIX_GUARD(s2n_hmac_update(mac, orig_write_ptr, data_bytes_to_take));
 
     /* Write the digest */
-    uint8_t *digest = s2n_stuffer_raw_write(&conn->out, mac_digest_size);
+    uint8_t *digest = s2n_stuffer_raw_write(&record_stuffer, mac_digest_size);
     POSIX_ENSURE_REF(digest);
 
     POSIX_GUARD(s2n_hmac_digest(mac, digest, mac_digest_size));
@@ -424,7 +456,7 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
 
     /* Write content type for TLS 1.3 record (RFC 8446 Section 5.2) */
     if (is_tls13_record) {
-        POSIX_GUARD(s2n_stuffer_write_uint8(&conn->out, content_type));
+        POSIX_GUARD(s2n_stuffer_write_uint8(&record_stuffer, content_type));
     }
 
     if (cipher_suite->record_alg->cipher->type == S2N_CBC) {
@@ -432,20 +464,20 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
          * include an extra padding length byte, also with the value 'p'.
          */
         for (int i = 0; i <= padding; i++) {
-            POSIX_GUARD(s2n_stuffer_write_uint8(&conn->out, padding));
+            POSIX_GUARD(s2n_stuffer_write_uint8(&record_stuffer, padding));
         }
     }
 
     /* Rewind to rewrite/encrypt the packet */
-    POSIX_GUARD(s2n_stuffer_rewrite(&conn->out));
+    POSIX_GUARD(s2n_stuffer_rewrite(&record_stuffer));
 
     /* Skip the header */
-    POSIX_GUARD(s2n_stuffer_skip_write(&conn->out, S2N_TLS_RECORD_HEADER_LENGTH));
+    POSIX_GUARD(s2n_stuffer_skip_write(&record_stuffer, S2N_TLS_RECORD_HEADER_LENGTH));
 
     uint16_t encrypted_length = data_bytes_to_take + mac_digest_size;
     switch (cipher_suite->record_alg->cipher->type) {
     case S2N_AEAD:
-        POSIX_GUARD(s2n_stuffer_skip_write(&conn->out, cipher_suite->record_alg->cipher->io.aead.record_iv_size));
+        POSIX_GUARD(s2n_stuffer_skip_write(&record_stuffer, cipher_suite->record_alg->cipher->io.aead.record_iv_size));
         encrypted_length += cipher_suite->record_alg->cipher->io.aead.tag_size;
         if (is_tls13_record) {
             /* one extra byte for content type */
@@ -455,7 +487,7 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
     case S2N_CBC:
         if (conn->actual_protocol_version > S2N_TLS10) {
             /* Leave the IV alone and unencrypted */
-            POSIX_GUARD(s2n_stuffer_skip_write(&conn->out, iv.size));
+            POSIX_GUARD(s2n_stuffer_skip_write(&record_stuffer, iv.size));
         }
         /* Encrypt the padding and the padding length byte too */
         encrypted_length += padding + 1;
@@ -471,19 +503,20 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
     }
 
     /* Check that stuffer have enough space to write encrypted record, because raw_write cannot expand tainted stuffer */
-    S2N_ERROR_IF(s2n_stuffer_space_remaining(&conn->out) < encrypted_length, S2N_ERR_RECORD_STUFFER_SIZE);
+    S2N_ERROR_IF(s2n_stuffer_space_remaining(&record_stuffer) < encrypted_length, S2N_ERR_RECORD_STUFFER_SIZE);
 
     /* Do the encryption */
-    struct s2n_blob en = { .size = encrypted_length, .data = s2n_stuffer_raw_write(&conn->out, encrypted_length) };
+    struct s2n_blob en = { .size = encrypted_length, .data = s2n_stuffer_raw_write(&record_stuffer, encrypted_length) };
     POSIX_GUARD(s2n_record_encrypt(conn, cipher_suite, session_key, &iv, &aad, &en, implicit_iv, block_size));
 
+    /* Sync the out stuffer write cursor with the record stuffer. */
+    POSIX_GUARD(s2n_stuffer_skip_write(&conn->out, s2n_stuffer_data_available(&record_stuffer)));
+
     if (conn->actual_protocol_version == S2N_TLS13 && content_type == TLS_CHANGE_CIPHER_SPEC) {
         conn->client = current_client_crypto;
         conn->server = current_server_crypto;
     }
 
-    conn->wire_bytes_out += actual_fragment_length + S2N_TLS_RECORD_HEADER_LENGTH;
-
     return data_bytes_to_take;
 }
 
