@@ -13,6 +13,8 @@
  * permissions and limitations under the License.
  */
 #include "crypto/s2n_fips.h"
+#include "crypto/s2n_libcrypto.h"
+#include "crypto/s2n_locking.h"
 
 #include "error/s2n_errno.h"
 
@@ -35,11 +37,6 @@
 
 static void s2n_cleanup_atexit(void);
 
-unsigned long s2n_get_openssl_version(void)
-{
-    return OPENSSL_VERSION_NUMBER;
-}
-
 static pthread_t main_thread = 0;
 static bool initialized = false;
 static bool atexit_cleanup = true;
@@ -52,8 +49,16 @@ int s2n_disable_atexit(void) {
 int s2n_init(void)
 {
     main_thread = pthread_self();
-    POSIX_GUARD(s2n_fips_init());
+    /* Should run before any init method that calls libcrypto methods
+     * to ensure we don't try to call methods that don't exist.
+     * It doesn't require any locks since it only deals with values that
+     * should be constant, so can run before s2n_locking_init. */
+    POSIX_GUARD_RESULT(s2n_libcrypto_validate_runtime());
+    /* Must run before any init method that allocates memory. */
     POSIX_GUARD(s2n_mem_init());
+    /* Must run before any init method that calls libcrypto methods. */
+    POSIX_GUARD_RESULT(s2n_locking_init());
+    POSIX_GUARD(s2n_fips_init());
     POSIX_GUARD_RESULT(s2n_rand_init());
     POSIX_GUARD(s2n_cipher_suites_init());
     POSIX_GUARD(s2n_security_policies_init());
@@ -83,11 +88,10 @@ static bool s2n_cleanup_atexit_impl(void)
     /* the configs need to be wiped before resetting the memory callbacks */
     s2n_wipe_static_configs();
 
-    bool a = s2n_result_is_ok(s2n_rand_cleanup_thread());
-    bool b = s2n_result_is_ok(s2n_rand_cleanup());
-    bool c = s2n_mem_cleanup() == 0;
-
-    return a && b && c;
+    return s2n_result_is_ok(s2n_rand_cleanup_thread()) &&
+           s2n_result_is_ok(s2n_rand_cleanup()) &&
+           s2n_result_is_ok(s2n_locking_cleanup()) &&
+           (s2n_mem_cleanup() == S2N_SUCCESS);
 }
 
 int s2n_cleanup(void)
