@@ -1050,7 +1050,7 @@ static void s_suback_multi_callback(
     /* Create list of (topic,qos) tuples */
     topic_qos_list = PyList_New(num_topics);
     if (!topic_qos_list) {
-        error_code = aws_py_raise_error();
+        error_code = aws_py_translate_py_error();
         goto done_prepping_args;
     }
 
@@ -1060,7 +1060,7 @@ static void s_suback_multi_callback(
 
         PyObject *tuple = Py_BuildValue("(s#i)", sub_i.topic.ptr, sub_i.topic.len, sub_i.qos);
         if (!tuple) {
-            error_code = aws_py_raise_error();
+            error_code = aws_py_translate_py_error();
             goto done_prepping_args;
         }
 
